@@ -26,8 +26,9 @@ struct fio_net_cmd {
 };
 
 enum {
-	FIO_SERVER_VER		= 1,
+	FIO_SERVER_VER		= 2,
 	FIO_SERVER_VER1		= 1,
+	FIO_SERVER_VER2		= 2,
 
 	FIO_SERVER_MAX_PDU	= 64,
 
@@ -87,63 +88,22 @@ extern int fio_clients_connect(void);
 extern int fio_clients_send_ini(const char *);
 extern int fio_handle_clients(void);
 extern void fio_client_add(const char *);
-extern void fio_client_add_cmd_option(const char *, const char *);
+extern int fio_client_add_cmd_option(const char *, const char *);
 
 extern int fio_recv_data(int sk, void *p, unsigned int len);
 extern int fio_send_data(int sk, const void *p, unsigned int len);
 extern void fio_net_cmd_crc(struct fio_net_cmd *);
-extern struct fio_net_cmd *fio_net_recv_cmd(int sk, int block);
+extern struct fio_net_cmd *fio_net_recv_cmd(int sk);
 
 extern int exit_backend;
 extern int fio_net_port;
 
-#ifdef FIO_LITTLE_ENDIAN
-#define __le16_to_cpu(x)		(x)
-#define __le32_to_cpu(x)		(x)
-#define __le64_to_cpu(x)		(x)
-#define __cpu_to_le16(x)		(x)
-#define __cpu_to_le32(x)		(x)
-#define __cpu_to_le64(x)		(x)
-#else
-#define __le16_to_cpu(x)		fio_swap16(x)
-#define __le32_to_cpu(x)		fio_swap32(x)
-#define __le64_to_cpu(x)		fio_swap64(x)
-#define __cpu_to_le16(x)		fio_swap16(x)
-#define __cpu_to_le32(x)		fio_swap32(x)
-#define __cpu_to_le64(x)		fio_swap64(x)
-#endif
-
-#define le16_to_cpu(val) ({			\
-	uint16_t *__val = &(val);		\
-	__le16_to_cpu(*__val);			\
-})
-#define le32_to_cpu(val) ({			\
-	uint32_t *__val = &(val);		\
-	__le32_to_cpu(*__val);			\
-})
-#define le64_to_cpu(val) ({			\
-	uint64_t *__val = &(val);		\
-	__le64_to_cpu(*__val);			\
-})
-#define cpu_to_le16(val) ({			\
-	uint16_t *__val = &(val);		\
-	__cpu_to_le16(*__val);			\
-})
-#define cpu_to_le32(val) ({			\
-	uint32_t *__val = &(val);		\
-	__cpu_to_le32(*__val);			\
-})
-#define cpu_to_le64(val) ({			\
-	uint64_t *__val = &(val);		\
-	__cpu_to_le64(*__val);			\
-})
-
 static inline void fio_init_net_cmd(struct fio_net_cmd *cmd, uint16_t opcode,
 				    const void *pdu, uint32_t pdu_len)
 {
 	memset(cmd, 0, sizeof(*cmd));
 
-	cmd->version	= __cpu_to_le16(FIO_SERVER_VER1);
+	cmd->version	= __cpu_to_le16(FIO_SERVER_VER2);
 	cmd->opcode	= cpu_to_le16(opcode);
 
 	if (pdu) {
